#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Un exemple simple de régression linéaire
# 
# Un jeu de données, une régression linéaire et quelques trucs bizarres. Le jeu de données est [Wine Data Set ](https://archive.ics.uci.edu/ml/datasets/wine).

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[2]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Récupération des données
# 
# Le premier code récupère les données depuis Internet.

# In[3]:


import pandas


# In[4]:


df = pandas.read_csv("https://archive.ics.uci.edu/ml/machine-learning-databases/wine/wine.data", 
                     header=None)
df.head(n=1)


# In[5]:


df.shape


# On récupère le nom des colonnes.

# In[6]:


s = "index Alcohol Malicacid Ash Alcalinityofash Magnesium Totalphenols Flavanoids"
s += " Nonflavanoidphenols Proanthocyanins Colorintensity Hue"
s += " OD280/OD315 Proline"
df.columns = s.split()
df.head().T


# In[7]:


df.describe().T


# ## Corrélations
# 
# On utilise le module [seaborn](https://seaborn.pydata.org/index.html) pour regarder visuellement les corrélations avec le graphe [Scatterplot Matrix](https://seaborn.pydata.org/examples/scatterplot_matrix.html).

# In[8]:


import seaborn as sns
sns.set(style="ticks")

sns.pairplot(df);


# Il y a quelques corrélations.

# ## Première régression
# 
# On essaye de prédire le taux d'alcool du vin à partir des autres variables. 

# In[9]:


X = df.drop(["Alcohol", "index"], axis=1)
y = df["Alcohol"]


# On divise entre train et test pour éviter l'overfitting.

# In[10]:


from sklearn.model_selection import train_test_split


# In[11]:


X_train, X_test, y_train, y_test = train_test_split(X, y)


# In[12]:


from sklearn.linear_model import LinearRegression


# In[13]:


model = LinearRegression()


# In[14]:


model.fit(X_train, y_train)


# In[15]:


model.coef_


# In[16]:


from sklearn.metrics import r2_score


# In[17]:


y_pred = model.predict(X_test)
r2_score(y_test, y_pred)


# In[18]:


y_predt = model.predict(X_train)
r2_score(y_train, y_predt)


# Même chose avec un arbre de décision.

# In[19]:


from sklearn.tree import DecisionTreeRegressor


# In[20]:


dt = DecisionTreeRegressor(min_samples_leaf=10)


# In[21]:


dt.fit(X_train, y_train)


# In[22]:


y_pred = dt.predict(X_test)
r2_score(y_test, y_pred)


# In[23]:


y_predt = dt.predict(X_train)
r2_score(y_train, y_predt)


# In[24]:


X_train.shape


# On vérifie avec une validation croisée.

# In[25]:


from sklearn.model_selection import cross_val_score


# In[26]:


cross_val_score(model, X, y, cv=5)


# Ca oscille trop pour que cela soit vraisemblable. C'est probablement dû à l'ordre dans les données.

# In[27]:


try:
    from sklearn.utils import shuffle
    df2 = shuffle(df).reset_index()
except Exception:
    import random
    index = list(df.index)
    df2 = df.iloc[index].reset_index()


# In[28]:


X2 = df2.drop("Alcohol", axis=1)
y2 = df2.Alcohol
cross_val_score(model, X2, y2, cv=5)


# Beaucoup plus stable.

# ## Feature importance

# In[29]:


dt.feature_importances_


# In[30]:


ft = pandas.DataFrame(dict(name=X_train.columns, fi=dt.feature_importances_))
ft = ft.sort_values("fi", ascending=False)
ft


# In[31]:


ft.set_index("name").plot(kind="bar");


# L'intensité de la couleur a l'air de jouer un grand rôle mais dans quel sens ?

# In[32]:


ax = df[["Colorintensity", "Alcohol"]].plot(x="Colorintensity", y="Alcohol", kind="scatter")
ax.set_title("Lien entre le degré d'alcool\net l'intensité de la couleur");


# La relation n'est pas linéaire car le degré d'alcool montre rarement au-dessus de 14.5 mais elle paraît linéaire par morceaux.

# In[33]: